;  ***************************************************************************
;  *                                                                         *
;  * File:   CPUID   .ASM         Version  1.10            Date: 1996-07-30  *
;  *                                                                         *
;  * Copyright 1991-1996 by kostis@acm.org (Kosta Kostis)                    *
;  *                                                                         *
;  ***************************************************************************
;  *                                                                         *
;  * Function:                                                               *
;  *     - returns CPU type                                                  *
;  *                                                                         *
;  ***************************************************************************
;  *                                                                         *
;  * Parameters:                                                             *
;  *     - none                                                              *
;  *                                                                         *
;  ***************************************************************************
;  *                                                                         *
;  * History:                                                                *
;  *     1996-07-30: KK V1.10                                                *
;  *        - added test for Pentium                                         *
;  *        - changed return codes (simplified)                              *
;  *     1996-07-26: KK V1.02                                                *
;  *        - minor cosmetical changes                                       *
;  *     1993-09-05: KK V1.01                                                *
;  *        - added test for i486                                            *
;  *     1991-11-15: KK V1.00                                                *
;  *        - initial coding                                                 *
;  *                                                                         *
;  ***************************************************************************
;
; This small assembler function checks the CPU type and returns an int value
; It's based on a sample program from Michael Tischer which was based on a
; sample program from Intel and has been converted into it's current form by
; kostis@acm.org (Kosta Kostis)
;
; int	CPUid (void)
;

.MODEL		SMALL,C

.DATA

; code values for CPUs

P_PentiumPro	EQU	6
P_Pentium	EQU	5
P_i486		EQU	4
P_i386		EQU	3
P_80286		EQU	2
P_80186		EQU	1
P_8086		EQU	0

P_FAM_MASK	EQU	0F00h
P_FAM_SHIFT	EQU	8

M_CPUID		MACRO
		DB	0fh	; hardcoded opcode for CPUID instruction
		DB	0a2h	;
		ENDM

; == Main Prog ==========================================================

.CODE

PUBLIC		CPUid			; int	CPUid (void)

CPUid		PROC

		pushf			; save flag register content

; -- check for i386/80286  -----------------------------------

		xor	ax,ax		; set AX to 0
		push	ax		; push on stack
		popf			; restore as flag register
		pushf			; push on stack again
		pop	ax		; restore as AX
		and	ax,0F000h	; keep bits 15 to 12
		cmp	ax,0F000h	; are all bits set?
		je	not_286		; skip to below 80286

; -- distinguish between i386 and 80286  ---------------------

is_286:		mov	dl,P_80286	; let's assume it's 80286
		mov	ax,07000h	;
		push	ax		; push 0x7000 on stack
		popf                    ; restore as flag register
		pushf                   ; push on stack
		pop	ax		; restore as AX
		and	ax,07000h	; keep bits 14 to 12
		je	pend		; are all bits cleared? -> 80286

; -- distinguish between i386 and i486 ------------------------

		mov	bx,sp		; save original value
		and	sp,not 3	; make sure we don't generate
					; an alignment fault

.386

		mov	dl,P_i386	; we got i386 or above

		pushfd			; check out EFLAG register
		pop	eax		;

		mov	ecx,eax		; make a safe copy in ecx
		xor	eax,40000h	; try to toggle bit #18 (AC)

		push	eax		; from eax to EFLAGS and back
		popfd			;
		pushfd			;
		pop	eax		;

		xor	eax,ecx		; check AC bit
		jz	done_386	; if it can't be changed, it's i386

; -- let's check for CPUID instruction ------------------------

		mov	dl,P_i486	; it's i486 or better

		mov	eax,ecx		; get original EFLAGS
		xor	eax,200000h	; try to toggle bit #21

		push	eax		; from eax to EFLAGS and back
		popfd			;
		pushfd			;
		pop	eax		;

		xor	eax,ecx		; check ID bit
		jz	done_386	; if it can't be changed, no CPUID

		push	ecx		; save ecx for now
		push	ebx		; and ebx also

		xor	eax,eax		; now retrieve information
		M_CPUID			; CPUID op code
		cmp	eax,1		; is 1 a valid input?
		jl	done_486	; no, let's leave it alone

		xor	eax,eax		;
		inc	eax		; eax=1, get family, model, stepping
		M_CPUID			; CPUID op code
		and	eax,P_FAM_MASK	; get CPU family (4-6)
		shr	eax,P_FAM_SHIFT	; shift it to the correct position
		mov	edx,eax		; copy value to DL

done_486:	pop	ebx		; restore ecx and ebx
		pop	ecx		;

done_386:	push	ecx		; restore flags to original state
		popfd			;

.8086
		mov	sp,bx		; restore stack pointer

		jmp	pend		; test finished

; -- check for 80186 or 8086 --------------------------------

not_286:	mov	dl,P_80186	; load code for 80186
		mov	al,0ffh		; set all bits in AL
		mov	cl,021h		; set # of shift ops in CL
		shr	al,cl		; shift right AL, CL times
		jne	pend		; must be 80188 or 80186

		mov	dl,P_8086	; no --> must be an 8088 or 8086
                                                                        
pend:		popf			; restore inital flag register
		xor	ah,ah		;
		mov	al,dl		; cpu id is function's return value

		ret			;

CPUid		ENDP			;

		END			;
